@@ -2,24 +2,19 @@
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
  * Copyright 2002-2007 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is  hereby granted without fee, provided that
- * the  above copyright   notice appear  in   all  copies and  that both  that
- * copyright  notice   and   this  permission   notice  appear  in  supporting
- * documentation, and that   the  name of  Frederic   Lepied not  be  used  in
- * advertising or publicity pertaining to distribution of the software without
- * specific,  written      prior  permission.     Frederic  Lepied   makes  no
- * representations about the suitability of this software for any purpose.  It
- * is provided "as is" without express or implied warranty.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * FREDERIC  LEPIED DISCLAIMS ALL   WARRANTIES WITH REGARD  TO  THIS SOFTWARE,
- * INCLUDING ALL IMPLIED   WARRANTIES OF MERCHANTABILITY  AND   FITNESS, IN NO
- * EVENT  SHALL FREDERIC  LEPIED BE   LIABLE   FOR ANY  SPECIAL, INDIRECT   OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA  OR PROFITS, WHETHER  IN  AN ACTION OF  CONTRACT,  NEGLIGENCE OR OTHER
- * TORTIOUS  ACTION, ARISING    OUT OF OR   IN  CONNECTION  WITH THE USE    OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
  *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software 
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #include "xf86Wacom.h"
@@ -75,26 +70,31 @@ static void xf86WcmSetScreen(LocalDevicePtr local, int *value0, int *value1)
 	double sizeX = priv->bottomX - priv->topX - 2*priv->tvoffsetX;
 	double sizeY = priv->bottomY - priv->topY - 2*priv->tvoffsetY;
 
-	DBG(6, ErrorF("xf86WcmSetScreen v0=%d v1=%d\n", *value0, *value1));
+	DBG(6, priv->debugLevel, ErrorF("xf86WcmSetScreen "
+		"v0=%d v1=%d\n", *value0, *value1));
 
 	if (!(local->flags & (XI86_ALWAYS_CORE | XI86_CORE_POINTER))) return;
 
 	if (!(priv->flags & ABSOLUTE_FLAG))
 	{
-		/* screenToSet lags by one event, but not that important */
+#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
 		priv->currentScreen = miPointerCurrentScreen()->myNum;
+#else
+		priv->currentScreen = miPointerGetScreen(local->dev)->myNum;
+#endif
 		for (i = 0; i < priv->numScreen; i++)
 			totalWidth += screenInfo.screens[i]->width;
 
 		maxHeight = screenInfo.screens[priv->currentScreen]->height;
 		priv->factorX = totalWidth / sizeX;
 		priv->factorY = maxHeight / sizeY;
-		DBG(10, ErrorF("xf86WcmSetScreen current=%d ToSet=%d\n", 
-				priv->currentScreen, screenToSet));
+		DBG(10, priv->debugLevel, ErrorF(
+			"xf86WcmSetScreen current=%d ToSet=%d\n", 
+			priv->currentScreen, screenToSet));
 		return;
 	}
 
-	if (priv->twinview == TV_NONE && (priv->flags & ABSOLUTE_FLAG))
+	if (priv->twinview == TV_NONE)
 	{
 		v0 = v0 > priv->bottomX ? priv->bottomX - priv->topX :
 			v0 < priv->topX ? 0 : v0 - priv->topX;
@@ -107,7 +107,7 @@ static void xf86WcmSetScreen(LocalDevicePtr local, int *value0, int *value1)
 	 */
 	if (screenInfo.numScreens == 1 || !priv->common->wcmMMonitor)
 	{
-		if (priv->twinview != TV_NONE && (priv->flags & ABSOLUTE_FLAG))
+		if (priv->twinview != TV_NONE)
 		{
 			if (priv->screen_no == -1)
 			{
@@ -134,8 +134,13 @@ static void xf86WcmSetScreen(LocalDevicePtr local, int *value0, int *value1)
 		else
 		{
 			/* tool on the tablet when driver starts */
+#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
 			if (miPointerCurrentScreen())
 				priv->currentScreen = miPointerCurrentScreen()->myNum;
+#else
+			if (miPointerGetScreen(local->dev))
+				priv->currentScreen = miPointerGetScreen(local->dev)->myNum;
+#endif
 			priv->factorX = screenInfo.screens[priv->currentScreen]->width / sizeX;
 			priv->factorY = screenInfo.screens[priv->currentScreen]->height / sizeY;
 		}
@@ -178,7 +183,8 @@ static void xf86WcmSetScreen(LocalDevicePtr local, int *value0, int *value1)
 		y = screenInfo.screens[screenToSet]->height - 1;
 
 	xf86XInputSetScreen(local, screenToSet, x, y);
-	DBG(10, ErrorF("xf86WcmSetScreen current=%d ToSet=%d\n", 
+	DBG(10, priv->debugLevel, ErrorF("xf86WcmSetScreen"
+		" current=%d ToSet=%d\n", 
 		priv->currentScreen, screenToSet));
 	priv->currentScreen = screenToSet;
 }
@@ -195,7 +201,8 @@ static void xf86WcmSendButtons(LocalDevicePtr local, int buttons, int rx, int ry
 	int button, mask, bsent = 0;
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
 	WacomCommonPtr common = priv->common;
-	DBG(6, ErrorF("xf86WcmSendButtons buttons=%d for %s\n", buttons, local->name));
+	DBG(6, priv->debugLevel, ErrorF("xf86WcmSendButtons "
+		"buttons=%d for %s\n", buttons, local->name));
 
 	/* Tablet PC buttons. */
 	if ( common->wcmTPCButton && !IsCursor(priv) && !IsPad(priv) && !IsEraser(priv) )
@@ -338,7 +345,6 @@ static void emitKeysym (DeviceIntPtr keydev, int keysym, int state)
 	/* Now that we have the keycode look for key index */
 	KeySymsRec *ksr = &keydev->key->curKeySyms;
 
-	DBG(10, ErrorF("emitKeysym keySym=%x state=%d\n", keysym, state));
 	for (i = ksr->minKeyCode; i <= ksr->maxKeyCode; i++)
 		if (ksr->map [(i - ksr->minKeyCode) * ksr->mapWidth] == keysym)
 			break;
@@ -406,7 +412,6 @@ static int WcmIsModifier(int keysym)
 			match = 1;
 			break;
 		}
-	DBG(10, ErrorF("WcmIsModifier keySym=%x match=%d\n", keysym, match));
 	return match;
 }
 
@@ -416,7 +421,9 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
 	WacomCommonPtr common = priv->common;
 	int is_absolute = priv->flags & ABSOLUTE_FLAG;
-	int is_core = local->flags & XI86_ALWAYS_CORE;
+#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
+	int is_core = local->flags & (XI86_ALWAYS_CORE | XI86_CORE_POINTER);
+#endif
 	int i, button_idx, naxes = priv->naxes;
 
 	if (IsPad(priv))
@@ -431,16 +438,20 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 
 	button_idx = button;
 	button = priv->button[button];
-	if (!button)
+	if (!button)  /* ignore this button event */
 		return;
 
+#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
 	/* Switch the device to core mode, if required */
 	if (!is_core && (button & AC_CORE))
 		xf86XInputSetSendCoreEvents (local, TRUE);
+#endif
 
-	DBG(4, ErrorF("sendAButton TPCButton(%s) button=%d state=%d" 
-		"code=%08x, for %s coreEvent=%s \n", common->wcmTPCButton ? 
-		"on" : "off", button_idx, mask, button, 
+	DBG(4, priv->debugLevel, ErrorF(
+		"sendAButton TPCButton(%s) button=%d state=%d " 
+		"code=%08x, for %s coreEvent=%s \n", 
+		common->wcmTPCButton ? "on" : "off", 
+		button_idx, mask, button, 
 		local->name, (button & AC_CORE) ? "yes" : "no"));
 
 	switch (button & AC_TYPE)
@@ -450,8 +461,15 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 */		/* Dynamically modify the button map as required --
 		 * to be moved in the place where button mappings are changed
 		 */
+#if defined WCM_XORG && GET_ABI_MAJOR(ABI_XINPUT_VERSION) > 0
+		/* +1 to be able to support Xorg xserver 1.4, which due to a
+		 * button translation bug will run the translation twice. By
+		 * adding 1 to the button# we're able to support at least a
+		 * simple button1 is button1 mapping. 
+		 */
+		button_idx++;
+#endif
 		local->dev->button->map [button_idx] = button & AC_CODE;
-
 		xf86PostButtonEvent(local->dev, is_absolute, button_idx,
 			mask != 0,0,naxes,rx,ry,rz,v3,v4,v5);
 
@@ -496,9 +514,16 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 
 	case AC_DBLCLICK:
 		/* Dynamically modify the button map as required --
-		 * to be moved in the place where button mappings are changed
+		 * to be moved in the place where button mappings are changed.
+		 * Only left double is supported.
 		 */
-		local->dev->button->map [button_idx] = button & AC_CODE;
+#if defined WCM_XORG && GET_ABI_MAJOR(ABI_XINPUT_VERSION) > 0
+		/* Match the button sent to the actual pos due to a bug in Xorg
+		 * xserver 1.4. I.e. button1 is button1.
+		 */
+		button_idx = 1;
+#endif
+		local->dev->button->map [button_idx] = 1;
 
 		if (mask)
 		{
@@ -519,10 +544,11 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 		break;
 	}
 
-	/* Switch the device out of the core mode, if required
-	 */
+#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
+	/* Switch the device out of the core mode, if required */
 	if (!is_core && (button & AC_CORE))
 		xf86XInputSetSendCoreEvents (local, FALSE);
+#endif
 }
 
 /*****************************************************************************
@@ -536,11 +562,12 @@ static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* d
 	int fakeButton = 0, i, value = 0, naxes = priv->naxes;
 	unsigned  *keyP = 0;
 	int is_absolute = priv->flags & ABSOLUTE_FLAG;
-	int is_core = local->flags & XI86_ALWAYS_CORE;
+#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
+	int is_core = local->flags & (XI86_ALWAYS_CORE | XI86_CORE_POINTER);
+#endif
+	DBG(10, priv->debugLevel, ErrorF("sendWheelStripEvents for %s \n", local->name));
 
-	DBG(10, ErrorF("sendWheelStripEvents for %s \n", local->name));
-
-	/* simulate events for relative wheel */
+	/* emulate events for relative wheel */
 	if ( ds->relwheel )
 	{
 		value = ds->relwheel;
@@ -556,8 +583,8 @@ static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* d
 		}
 	}
 
-	/* simulate events for absolute wheel when needed */
-/*	if ( ds->abswheel != priv->oldWheel )
+	/* emulate events for absolute wheel when needed */
+	if ( ds->abswheel != priv->oldWheel )
 	{
 		value = priv->oldWheel - ds->abswheel;
 		if ( value > 0 )
@@ -571,8 +598,8 @@ static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* d
 			keyP = priv->wdnk;
 		}
 	}
-*/
-	/* simulate events for left strip */
+
+	/* emulate events for left strip */
 	if ( ds->stripx != priv->oldStripX )
 	{
 		int temp = 0, n;
@@ -592,14 +619,14 @@ static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* d
 			fakeButton = priv->striplup;
 			keyP = priv->slupk;
 		}
-		else
+		else if ( value < 0 )
 		{
 			fakeButton = priv->stripldn;
 			keyP = priv->sldnk;
 		}
 	}
 
-	/* simulate events for right strip */
+	/* emulate events for right strip */
 	if ( ds->stripy != priv->oldStripY )
 	{
 		int temp = 0, n;
@@ -619,30 +646,42 @@ static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* d
 			fakeButton = priv->striprup;
 			keyP = priv->srupk;
 		}
-		else
+		else if ( value < 0 )
 		{
 			fakeButton = priv->striprdn;
 			keyP = priv->srdnk;
 		}
 	}
 
-	DBG(10, ErrorF("sendWheelStripEvents send fakeButton %x with value = %d \n", fakeButton, value));
-
 	if (!fakeButton) return;
 
+	DBG(10, priv->debugLevel, ErrorF("sendWheelStripEvents "
+		"send fakeButton %x with value = %d \n", 
+		fakeButton, value));
+
+#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
 	/* Switch the device to core mode, if required */
 	if (!is_core && (fakeButton & AC_CORE))
 		xf86XInputSetSendCoreEvents (local, TRUE);
-
+#endif
 	switch (fakeButton & AC_TYPE)
 	{
 	    case AC_BUTTON:
-		/* pad may only have 2 buttons */
-		local->dev->button->map [0] = fakeButton & AC_CODE;
-
-		xf86PostButtonEvent(local->dev, is_absolute, 0,
+		/* send both button on/off in the same event for pad */
+		i = 0;
+#if defined WCM_XORG && GET_ABI_MAJOR(ABI_XINPUT_VERSION) > 0
+		/* Match the button sent to the actual pos due to a bug in Xorg
+		 * xserver 1.4. I.e. button4 is button4. This makes it usable if
+		 * no exotic configuration is made.
+		 */
+		i = fakeButton & AC_CODE;
+#endif
+		local->dev->button->map [i] = fakeButton & AC_CODE;
+	
+		xf86PostButtonEvent(local->dev, is_absolute, i,
 			1,0,naxes,x,y,z,v3,v4,v5);
-		xf86PostButtonEvent(local->dev, is_absolute, 0,
+
+		xf86PostButtonEvent(local->dev, is_absolute, i,
 			0,0,naxes,x,y,z,v3,v4,v5);
 
 	    break;
@@ -670,10 +709,12 @@ static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* d
 		ErrorF ("WARNING: [%s] unsupported event %x \n", local->name, fakeButton);
 	}
 
+#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
 	/* Switch the device out of the core mode, if required
 	 */
 	if (!is_core && (fakeButton & AC_CORE))
 		xf86XInputSetSendCoreEvents (local, FALSE);
+#endif
 }
 
 /*****************************************************************************
@@ -695,8 +736,8 @@ static void sendCommonEvents(LocalDevicePtr local, const WacomDeviceState* ds, i
 	if (priv->oldButtons != buttons)
 		xf86WcmSendButtons(local,buttons,x,y,z,v3,v4,v5);
 
-	/* simulate wheel/strip events when defined */
-	if ( ds->relwheel || 
+	/* emulate wheel/strip events when defined */
+	if ( ds->relwheel || ds->abswheel || 
 		( (ds->stripx - priv->oldStripX) && ds->stripx && priv->oldStripX) || 
 			((ds->stripy - priv->oldStripY) && ds->stripy && priv->oldStripY) )
 		sendWheelStripEvents(local, ds, x, y, z, v3, v4, v5);
@@ -735,7 +776,8 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 
 	if (priv->serial && serial != priv->serial)
 	{
-		DBG(10, ErrorF("[%s] serial number is %d but your system configured %d", 
+		DBG(10, priv->debugLevel, ErrorF("[%s] serial number"
+			" is %d but your system configured %d", 
 			local->name, serial, (int)priv->serial));
 		return;
 	}
@@ -747,7 +789,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		ty = ds->stripy;
 	}
 
-	DBG(7, ErrorF("[%s] o_prox=%s x=%d y=%d z=%d "
+	DBG(7, priv->debugLevel, ErrorF("[%s] o_prox=%s x=%d y=%d z=%d "
 		"b=%s b=%d tx=%d ty=%d wl=%d rot=%d th=%d\n",
 		(type == STYLUS_ID) ? "stylus" :
 			(type == CURSOR_ID) ? "cursor" : 
@@ -787,7 +829,8 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 	}
 	v5 = wheel;
 
-	DBG(6, ErrorF("[%s] %s prox=%d\tx=%d\ty=%d\tz=%d\tv3=%d\tv4=%d\tv5=%d\tid=%d"
+	DBG(6, priv->debugLevel, ErrorF("[%s] %s prox=%d\tx=%d"
+		"\ty=%d\tz=%d\tv3=%d\tv4=%d\tv5=%d\tid=%d"
 		"\tserial=%d\tbutton=%s\tbuttons=%d\n",
 		local->name,
 		is_absolute ? "abs" : "rel",
@@ -805,19 +848,26 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		y = priv->topY;
 	priv->currentX = x;
 	priv->currentY = y;
+
+	/* update the old records */
+	if(!priv->oldProximity)
+	{
+		priv->oldWheel = wheel;
+		priv->oldX = priv->currentX;
+		priv->oldY = priv->currentY;
+		priv->oldZ = z;
+		priv->oldTiltX = tx;
+		priv->oldTiltY = ty;
+		priv->oldStripX = ds->stripx;
+		priv->oldStripY = ds->stripy;
+		priv->oldRot = rot;
+		priv->oldThrottle = throttle;
+	}
 	if (!is_absolute)
 	{
-		if(!priv->oldProximity)
-		{
-			/* don't move the cursor */
-			x = 0;
-			y = 0;
-		}
-		else
-		{
-			x -= priv->oldX;
-			y -= priv->oldY;
-		}
+		x -= priv->oldX;
+		y -= priv->oldY;
+
 		/* don't apply speed for fairly small increments */
 		no_jitter = (priv->speed*3 > 4) ? priv->speed*3 : 4;
 		relacc = (MAX_ACCEL-priv->accel)*(MAX_ACCEL-priv->accel);
@@ -846,6 +896,14 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 			y *= param > 20.00 ? 20.00 : param;
 		}		
 	}
+
+#if defined WCM_XORG && GET_ABI_MAJOR(ABI_XINPUT_VERSION) > 0
+	/* Ugly hack for Xorg 7.3, which doesn't call xf86WcmDevConvert
+	 * for coordinate conversion at the moment */
+	x = (int)((double)(x - priv->topX) * priv->factorX + 0.5);
+	y = (int)((double)(y - priv->topY) * priv->factorY + 0.5);
+#endif
+
 	if (type != PAD_ID)
 	{
 		/* coordinates are ready we can send events */
@@ -886,15 +944,18 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 	}
 	else
 	{
-		if (v3 || v4 || buttons || ds->relwheel)
+		if (v3 || v4 || v5 || buttons || ds->relwheel)
 		{
 			x = 0;
 			y = 0;
-			if ( v3 || v4 )
+			if ( v3 || v4 || v5 )
 				xf86WcmSetScreen(local, &x, &y);
 			sendCommonEvents(local, ds, x, y, z, v3, v4, v5);
 			is_proximity = 1;
-			if ( v3 || v4 )
+			/* xf86PostMotionEvent is only needed to post the valuators
+			 * It should NOT move the cursor.
+			 */
+			if ( v3 || v4 || v5 )
 			{
 	 			xf86PostMotionEvent(local->dev, is_absolute,
 					0, naxes, x, y, z, v3, v4, v5);
@@ -906,7 +967,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 				xf86WcmSendButtons(local, buttons, 
 					x, y, z, v3, v4, v5);
 			if (priv->oldProximity && local->dev->proximity)
- 				xf86PostProximityEvent(local->dev, is_absolute, 0, naxes, 
+ 				xf86PostProximityEvent(local->dev, 0, 0, naxes, 
 				x, y, z, v3, v4, v5);
 			is_proximity = 0;
 		}
@@ -946,39 +1007,47 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 
 /*****************************************************************************
  * xf86WcmSuppress --
- *  Determine whether device state has changed enough - return 1
+ *  Determine whether device state has changed enough - return 0
  *  if not.
  ****************************************************************************/
 
-static int xf86WcmSuppress(int suppress, const WacomDeviceState* dsOrig,
-	const WacomDeviceState* dsNew)
+static int xf86WcmSuppress(WacomCommonPtr common, const WacomDeviceState* dsOrig, 
+	WacomDeviceState* dsNew)
 {
+	int suppress = common->wcmSuppress;
 	/* NOTE: Suppression value of zero disables suppression. */
-	DBG(11, ErrorF("xf86WcmSuppress checking data (suppress=%d)\n", suppress));
+	int returnV = 0;
 
-	if (dsOrig->buttons != dsNew->buttons) return 0;
-	if (dsOrig->proximity != dsNew->proximity) return 0;
-	if (dsOrig->stripx != dsNew->stripx) return 0;
-	if (dsOrig->stripy != dsNew->stripy) return 0;
-	if (ABS(dsOrig->tiltx - dsNew->tiltx) > suppress) return 0;
-	if (ABS(dsOrig->tilty - dsNew->tilty) > suppress) return 0;
-	if (ABS(dsOrig->pressure - dsNew->pressure) > suppress) return 0;
-	if (ABS(dsOrig->throttle - dsNew->throttle) > suppress) return 0;
+	if (dsOrig->buttons != dsNew->buttons) returnV = 1;
+	if (dsOrig->proximity != dsNew->proximity) returnV = 1;
+	if (dsOrig->stripx != dsNew->stripx) returnV = 1;
+	if (dsOrig->stripy != dsNew->stripy) returnV = 1;
+	if (ABS(dsOrig->tiltx - dsNew->tiltx) > suppress) returnV = 1;
+	if (ABS(dsOrig->tilty - dsNew->tilty) > suppress) returnV = 1;
+	if (ABS(dsOrig->pressure - dsNew->pressure) > suppress) returnV = 1;
+	if (ABS(dsOrig->throttle - dsNew->throttle) > suppress) returnV = 1;
 	if (ABS(dsOrig->rotation - dsNew->rotation) > suppress &&
-		(1800 - ABS(dsOrig->rotation - dsNew->rotation)) >  suppress) return 0;
+		(1800 - ABS(dsOrig->rotation - dsNew->rotation)) >  suppress) returnV = 1;
 
-	/* look for change in absolute wheel
-	 * position or any relative wheel movement */
-	if ((ABS(dsOrig->abswheel - dsNew->abswheel) > suppress) ||
-		(dsNew->relwheel != 0)) return 0;
+	/* look for change in absolute wheel position 
+	 * or any relative wheel movement
+	 */
+	if ((ABS(dsOrig->abswheel - dsNew->abswheel) > suppress) 
+		|| (dsNew->relwheel != 0)) returnV = 1;
 
 	/* need to check if cursor moves or not */
 	if ((ABS(dsOrig->x - dsNew->x) > suppress) || 
-		(ABS(dsOrig->y - dsNew->y) > suppress)) return 2;
+			(ABS(dsOrig->y - dsNew->y) > suppress)) 
+		returnV = 2;
+	else if (returnV == 1) /* don't move the cursor */
+	{
+		dsNew->x = dsOrig->x;
+		dsNew->y = dsOrig->y;
+	}
 
-	/* discard this event */
-	DBG(11, ErrorF("xf86WcmSuppress discarded data\n"));
-	return 1;
+	DBG(10, common->debugLevel, ErrorF("xf86WcmSuppress at level = %d"
+		" return value = %d\n", suppress, returnV));
+	return returnV;
 }
 
 /*****************************************************************************
@@ -991,7 +1060,7 @@ Bool xf86WcmOpen(LocalDevicePtr local)
 	WacomCommonPtr common = priv->common;
 	WacomDeviceClass** ppDevCls;
 
-	DBG(1, ErrorF("opening %s\n", common->wcmDevice));
+	DBG(1, priv->debugLevel, ErrorF("opening %s\n", common->wcmDevice));
 
 	local->fd = xf86WcmOpenTablet(local);
 	if (local->fd < 0)
@@ -1042,30 +1111,25 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 	WacomFilterState* fs;
 	int i, suppress = 0;
 
-	/* tool on the tablet when driver starts */
-	if (!miPointerCurrentScreen())
-	{
-		DBG(6, ErrorF("xf86WcmEvent: Wacom driver can not get Current Screen ID\n"));
-		DBG(6, ErrorF("Please remove Wacom tool from the tablet.\n"));
-		return;
-	}
+	pChannel = common->wcmChannel + channel;
+	pLast = &pChannel->valid.state;
+
+	DBG(10, common->debugLevel, ErrorF("xf86WcmEvent at channel = %d\n", channel));
 
 	/* sanity check the channel */
 	if (channel >= MAX_CHANNELS)
 		return;
 	
-	pChannel = common->wcmChannel + channel;
-	pLast = &pChannel->valid.state;
-
 	/* we must copy the state because certain types of filtering
 	 * will need to change the values (ie. for error correction) */
 	ds = *pState;
 
 	/* timestamp the state for velocity and acceleration analysis */
 	ds.sample = (int)GetTimeInMillis();
-
-	DBG(10, ErrorF("xf86WcmEvent: c=%d i=%d t=%d s=%u x=%d y=%d b=%d "
-		"p=%d rz=%d tx=%d ty=%d aw=%d rw=%d t=%d df=%d px=%d st=%d\n",
+	DBG(10, common->debugLevel, ErrorF("xf86WcmEvent: "
+		"c=%d i=%d t=%d s=%u x=%d y=%d b=%d "
+		"p=%d rz=%d tx=%d ty=%d aw=%d rw=%d "
+		"t=%d df=%d px=%d st=%d cs=%d \n",
 		channel,
 		ds.device_id,
 		ds.device_type,
@@ -1073,13 +1137,16 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 		ds.x, ds.y, ds.buttons,
 		ds.pressure, ds.rotation, ds.tiltx,
 		ds.tilty, ds.abswheel, ds.relwheel, ds.throttle,
-		ds.discard_first, ds.proximity, ds.sample));
+		ds.discard_first, ds.proximity, ds.sample,
+		pChannel->nSamples));
 
 #ifdef LINUX_INPUT
 	/* Discard the first 2 USB packages due to events delay */
 	if ( (pChannel->nSamples < 2) && (common->wcmDevCls == &gWacomUSBDevice) && ds.device_type != PAD_ID )
 	{
-		DBG(11, ErrorF("discarded %dth USB data.\n", pChannel->nSamples));
+		DBG(11, common->debugLevel, 
+			ErrorF("discarded %dth USB data.\n", 
+			pChannel->nSamples));
 		++pChannel->nSamples;
 		return; /* discard */
 	}
@@ -1087,9 +1154,9 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 	fs = &pChannel->rawFilter;
 	if (!fs->npoints && ds.proximity)
 	{
-		DBG(11, ErrorF("initialize Channel data.\n"));
+		DBG(11, common->debugLevel, ErrorF("initialize Channel data.\n"));
 		/* store channel device state for later use */
-		for (i=MAX_SAMPLES - 1; i>=0; i--)
+		for (i=common->wcmRawSample - 1; i>=0; i--)
 		{
 			fs->x[i]= ds.x;
 			fs->y[i]= ds.y;
@@ -1099,7 +1166,7 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 		++fs->npoints;
 	} else  {
 		/* Filter raw data, fix hardware defects, perform error correction */
-		for (i=MAX_SAMPLES - 1; i>0; i--)
+		for (i=common->wcmRawSample - 1; i>0; i--)
 		{
 			fs->x[i]= fs->x[i-1];
 			fs->y[i]= fs->y[i-1];
@@ -1108,7 +1175,7 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 		fs->y[0] = ds.y;
 		if (HANDLE_TILT(common) && (ds.device_type == STYLUS_ID || ds.device_type == ERASER_ID))
 		{
-			for (i=MAX_SAMPLES - 1; i>0; i--)
+			for (i=common->wcmRawSample - 1; i>0; i--)
 			{
 				fs->tiltx[i]= fs->tiltx[i-1];
 				fs->tilty[i]= fs->tilty[i-1];
@@ -1116,19 +1183,20 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 			fs->tiltx[0] = ds.tiltx;
 			fs->tilty[0] = ds.tilty;
 		}
-		if (RAW_FILTERING(common) && common->wcmModel->FilterRaw)
+		if (RAW_FILTERING(common) && common->wcmModel->FilterRaw && ds.device_type != PAD_ID)
 		{
 			if (common->wcmModel->FilterRaw(common,pChannel,&ds))
 			{
-				DBG(10, ErrorF("Raw filtering discarded data.\n"));
+				DBG(10, common->debugLevel, ErrorF(
+					"Raw filtering discarded data.\n"));
 				resetSampleCounter(pChannel);
 				return; /* discard */
 			}
 		}
 
 		/* Discard unwanted data */
-		suppress = xf86WcmSuppress(common->wcmSuppress, pLast, &ds);
-		if (suppress == 1)
+		suppress = xf86WcmSuppress(common, pLast, &ds);
+		if (!suppress)
 		{
 			resetSampleCounter(pChannel);
 			return;
@@ -1144,9 +1212,9 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 	/* save channel device state and device to which last event went */
 	memmove(pChannel->valid.states + 1,
 		pChannel->valid.states,
-		sizeof(WacomDeviceState) * (MAX_SAMPLES - 1));
+		sizeof(WacomDeviceState) * (common->wcmRawSample - 1));
 	pChannel->valid.state = ds; /*save last raw sample */
-	if (pChannel->nSamples < MAX_SAMPLES) ++pChannel->nSamples;
+	if (pChannel->nSamples < common->wcmRawSample) ++pChannel->nSamples;
 
 	commonDispatchDevice(common,channel,pChannel, suppress);
 	resetSampleCounter(pChannel);
@@ -1198,10 +1266,9 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 	WacomToolPtr tool = NULL;
 	WacomToolPtr tooldef = NULL;
 	WacomDeviceState* ds = &pChannel->valid.states[0];
+	WacomDevicePtr priv = NULL;
 
-	DBG(10, ErrorF("commonDispatchEvents\n"));
-
-	if (!ds->device_type)
+	if (!ds->device_type && ds->proximity)
 	{
 		/* Tool may be on the tablet when X starts. 
 		 * Figure out device type by device id
@@ -1220,7 +1287,6 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 			default:
 				ds->device_type = idtotype(ds->device_id);
 		}
-		ds->proximity = 1;
 		if (ds->serial_num)
 			for (tool = common->wcmTool; tool; tool = tool->next)
 				if (ds->serial_num == tool->serial)
@@ -1230,6 +1296,7 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 				}
 	}
 
+	DBG(10, common->debugLevel, ErrorF("commonDispatchDevice device type = %d\n", ds->device_type));
 	/* Find the device the current events are meant for */
 	/* 1: Find the tool (the one with correct serial or in second
 	 * hand, the one with serial set to 0 if no match with the
@@ -1256,7 +1323,8 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 		 * Skip this if only one area is defined 
 		 */
 		WacomToolAreaPtr outprox = NULL;
-		if (tool->current && tool->arealist->next && !xf86WcmPointInArea(tool->current, ds->x, ds->y))
+		if (tool->current && tool->arealist->next && 
+			!xf86WcmPointInArea(tool->current, ds->x, ds->y))
 		{
 			outprox = tool->current;
 			tool->current = NULL;
@@ -1287,7 +1355,7 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 				/* Send soft prox-out for the old area */
 				LocalDevicePtr oDev = outprox->device;
 				WacomDeviceState out = { 0 };
-				DBG(2, ErrorF("Soft prox-out for %s\n",
+				DBG(2, common->debugLevel, ErrorF("Soft prox-out for %s\n",
 					outprox->device->name));
 				xf86WcmSendEvents(oDev, &out);
 			}
@@ -1299,24 +1367,37 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 		if(tool->current)
 		{
 			pDev = tool->current->device;
-			DBG(11, ErrorF("tool id=%d for %s\n",
+			DBG(11, common->debugLevel, ErrorF("tool id=%d for %s\n",
 				       ds->device_type, pDev->name));
 		}
 	}
 	/* X: InputDevice selection done! */
 
-	DBG(11, ErrorF("commonDispatchEvents: %p \n",(void *)pDev));
+	/* Tool on the tablet when driver starts. This sometime causes
+	 * access errors to the device */
+#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
+	if (!miPointerCurrentScreen())
+#else
+	if (pDev && !miPointerGetScreen(pDev->dev))
+#endif
+	{
+		DBG(1, common->debugLevel, ErrorF("xf86WcmEvent: "
+			"Wacom driver can not get Current Screen ID\n"));
+		DBG(1, common->debugLevel, ErrorF(
+			"Please remove Wacom tool from the tablet.\n"));
+		return;
+	}
 
 	/* if a device matched criteria, handle filtering per device
 	 * settings, and send event to XInput */
 	if (pDev)
 	{
 		WacomDeviceState filtered = pChannel->valid.state;
-		WacomDevicePtr priv = pDev->private;
 
 		/* Device transformations come first */
 		/* button 1 Threshold test */
 		int button = 1;
+		priv = pDev->private;
 		if ( IsStylus(priv) || IsEraser(priv))
 		{
 			if (filtered.pressure < common->wcmThreshold )
@@ -1369,7 +1450,8 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 		}
 		else if (priv->throttleLimit < sampleTime)
 		{
-			DBG(6, ErrorF("LIMIT REACHED: s=%d l=%d n=%d v=%d "
+			DBG(6, priv->debugLevel, ErrorF("LIMIT REACHED: "
+				"s=%d l=%d n=%d v=%d "
 				"N=%d\n", priv->throttleStart,
 				priv->throttleLimit, sampleTime,
 				ds->throttle, sampleTime + ticks));
@@ -1397,7 +1479,8 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 
 			if (ABS(deltx)<1 && ABS(delty)<1) 
 			{
-				DBG(10, ErrorF("Ignore non-movement relative data \n"));
+				DBG(10, common->debugLevel, ErrorF(
+					"Ignore non-movement relative data \n"));
 				return;
 			}
 			else
@@ -1424,8 +1507,13 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 				if (common->wcmMaxCursorDist < filtered.distance)
 					common->wcmMaxCursorDist = filtered.distance;
 			}
-			DBG(10, ErrorF("Distance over the tablet: %d, ProxoutDist: %d current min/max %d hard prox: %d\n",
-				filtered.distance, common->wcmCursorProxoutDist, common->wcmMaxCursorDist, ds->proximity));
+			DBG(10, common->debugLevel, ErrorF("Distance over"
+				" the tablet: %d, ProxoutDist: %d current"
+				" min/max %d hard prox: %d\n",
+				filtered.distance, 
+				common->wcmCursorProxoutDist, 
+				common->wcmMaxCursorDist, 
+				ds->proximity));
 
 			if (priv->oldProximity)
 			{
@@ -1451,7 +1539,8 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 	/* otherwise, if no device matched... */
 	else
 	{
-		DBG(11, ErrorF("no device matches with id=%d, serial=%d\n",
+		DBG(11, common->debugLevel, ErrorF("no device matches with"
+				" id=%d, serial=%d\n",
 				ds->device_type, ds->serial_num));
 	}
 }
@@ -1523,7 +1612,7 @@ int xf86WcmInitTablet(LocalDevicePtr local, WacomModelPtr model,
 	}
 
 	/* Enable hardware suppress, if requested and available */
-	if ((common->wcmSuppress != 0) && model->EnableSuppress)
+	if (model->EnableSuppress)
 	{
 		if (model->EnableSuppress(local) != Success)
 			return !Success;
@@ -1547,12 +1636,11 @@ int xf86WcmInitTablet(LocalDevicePtr local, WacomModelPtr model,
 	/* output tablet state as probed */
 	if (xf86Verbose)
 		ErrorF("%s Wacom %s tablet speed=%d maxX=%d maxY=%d maxZ=%d "
-			"resX=%d resY=%d suppress=%d tilt=%s\n",
+			"resX=%d resY=%d  tilt=%s\n",
 			XCONFIG_PROBED,
 			model->name, common->wcmLinkSpeed,
 			common->wcmMaxX, common->wcmMaxY, common->wcmMaxZ,
 			common->wcmResolX, common->wcmResolY,
-			common->wcmSuppress,
 			HANDLE_TILT(common) ? "enabled" : "disabled");
   
 	/* start the tablet data */
