@@ -2,24 +2,19 @@
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
  * Copyright 2002-2007 by Ping Cheng, Wacom. <pingc@wacom.com>
  *                                                                            
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is  hereby granted without fee, provided that
- * the  above copyright   notice appear  in   all  copies and  that both  that
- * copyright  notice   and   this  permission   notice  appear  in  supporting
- * documentation, and that   the  name of  Frederic   Lepied not  be  used  in
- * advertising or publicity pertaining to distribution of the software without
- * specific,  written      prior  permission.     Frederic  Lepied   makes  no
- * representations about the suitability of this software for any purpose.  It
- * is provided "as is" without express or implied warranty.                   
- *                                                                            
- * FREDERIC  LEPIED DISCLAIMS ALL   WARRANTIES WITH REGARD  TO  THIS SOFTWARE,
- * INCLUDING ALL IMPLIED   WARRANTIES OF MERCHANTABILITY  AND   FITNESS, IN NO
- * EVENT  SHALL FREDERIC  LEPIED BE   LIABLE   FOR ANY  SPECIAL, INDIRECT   OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA  OR PROFITS, WHETHER  IN  AN ACTION OF  CONTRACT,  NEGLIGENCE OR OTHER
- * TORTIOUS  ACTION, ARISING    OUT OF OR   IN  CONNECTION  WITH THE USE    OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software 
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #include "xf86Wacom.h"
@@ -38,18 +33,18 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 	WacomToolPtr     tool;
 	WacomToolAreaPtr area;
 
-	priv = (WacomDevicePtr) xalloc(sizeof(WacomDeviceRec));
+	priv = (WacomDevicePtr) xcalloc(1, sizeof(WacomDeviceRec));
 	if (!priv)
 		return NULL;
 
-	common = (WacomCommonPtr) xalloc(sizeof(WacomCommonRec));
+	common = (WacomCommonPtr) xcalloc(1, sizeof(WacomCommonRec));
 	if (!common)
 	{
 		xfree(priv);
 		return NULL;
 	}
 
-	tool = (WacomToolPtr) xalloc(sizeof(WacomTool));
+	tool = (WacomToolPtr) xcalloc(1, sizeof(WacomTool));
 	if(!tool)
 	{
 		xfree(priv);
@@ -57,7 +52,7 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 		return NULL;
 	}
 
-	area = (WacomToolAreaPtr) xalloc(sizeof(WacomToolArea));
+	area = (WacomToolAreaPtr) xcalloc(1, sizeof(WacomToolArea));
 	if(!area)
 	{
 		xfree(tool);
@@ -66,7 +61,7 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 		return NULL;
 	}
 
-	local = xf86AllocateInput(gWacomModule.v4.wcmDrv, 0);
+	local = xf86AllocateInput(gWacomModule.wcmDrv, 0);
 	if (!local)
 	{
 		xfree(area);
@@ -90,11 +85,14 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 	local->dev = NULL;
 	local->private = priv;
 	local->private_flags = 0;
+#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
 	local->history_size  = 0;
+#endif
 	local->old_x = -1;
 	local->old_y = -1;
-	
-	memset(priv,0,sizeof(*priv));
+
+	priv->next = NULL;
+	priv->local = local;
 	priv->flags = flag;          /* various flags (device type, absolute, first touch...) */
 	priv->oldX = 0;             /* previous X position */
 	priv->oldY = 0;             /* previous Y position */
@@ -128,35 +126,37 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 	priv->nPressCtrl [1] = 0;    /* pressure curve y0 */
 	priv->nPressCtrl [2] = 100;  /* pressure curve x1 */
 	priv->nPressCtrl [3] = 100;  /* pressure curve y1 */
-	/* Pad by default emits keypresses, other devices emits buttons */
+
+	/* Default button and expresskey values */
 	for (i=0; i<MAX_BUTTONS; i++)
 		priv->button[i] = IsPad (priv) ?
 			(AC_BUTTON | (MAX_MOUSE_BUTTONS/2 + i + 1)) : (AC_BUTTON | (i + 1));
-	for (i=MAX_MOUSE_BUTTONS/2; i<MAX_BUTTONS; i++)
+/*	for (i=MAX_MOUSE_BUTTONS/2; i<MAX_BUTTONS; i++)
 		priv->button[i] = IsPad (priv) ?
 			(AC_KEY | (XK_F1 + i)) : (AC_BUTTON | (i + 1));
-
+*/
 	/* Now for backward compatibility make some keys emit button events
 	 * with button indices 9-16...
 	 */
-	if (IsPad (priv) /* && check for backward-compatibility models? */)
+/*	if (IsPad (priv))
 		for (i = 0; i < 8; i++)
 			priv->button[i] = (AC_BUTTON | (i + 9));
-
+*/
 	for (i=0; i<MAX_BUTTONS; i++)
 		for (j=0; j<256; j++)
 			priv->keys[i][j] = 0;
 
 	priv->nbuttons = MAX_BUTTONS;      /* Default number of buttons */
-	priv->relup = 4;		   /* Default relative wheel up event */
-	priv->reldn = 5;		   /* Default relative wheel down event */
-	priv->wheelup = 4;		   /* Default absolute wheel up event */
-	priv->wheeldn = 5;		   /* Default absolute wheel down event */
-	priv->striplup = 0;		   /* Default left strip up event. Let user app take care of */
-	priv->stripldn = 0;		   /* Default left strip down event. Let user app take care of */
-	priv->striprup = 0;		   /* Default right strip up event. Let user app take care of */
-	priv->striprdn = 0;		   /* Default right strip down event. Let user app take care of */
+	priv->relup = 5;		   /* Default relative wheel up event */
+	priv->reldn = 4;		   /* Default relative wheel down event */
+	priv->wheelup = 0;		   /* Default absolute wheel up event */
+	priv->wheeldn = 0;		   /* Default absolute wheel down event */
+	priv->striplup = 4;		   /* Default left strip up event */
+	priv->stripldn = 5;		   /* Default left strip down event */
+	priv->striprup = 4;		   /* Default right strip up event */
+	priv->striprdn = 5;		   /* Default right strip down event */
 	priv->naxes = 6;                   /* Default number of axes */
+	priv->debugLevel = 0;              /* debug level */
 	priv->numScreen = screenInfo.numScreens; /* configured screens count */
 	priv->currentScreen = 0;                 /* current screen in display */
 
@@ -171,14 +171,10 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 	priv->throttleStart = 0;
 	priv->throttleLimit = -1;
 	
-	memset(common,0,sizeof(*common));
-	memset(common->wcmChannel, 0, sizeof(common->wcmChannel));
 	common->wcmDevice = "";                  /* device file name */
-	common->wcmSuppress = DEFAULT_SUPPRESS;  /* transmit position if increment is superior */
 	common->wcmFlags = RAW_FILTERING_FLAG;   /* various flags */
-	common->wcmDevices = (LocalDevicePtr*) xalloc(sizeof(LocalDevicePtr));
-	common->wcmDevices[0] = local;
-	common->wcmNumDevices = 1;         /* number of devices */
+	common->wcmDevices = priv;
+	common->npadkeys = 0;		   /* Default number of pad keys */
 	common->wcmMaxX = 0;               /* max X value */
 	common->wcmMaxY = 0;               /* max Y value */
 	common->wcmMaxZ = 0;               /* max Z value */
@@ -191,6 +187,8 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 	common->wcmThreshold = 0;       /* unconfigured threshold */
 	common->wcmLinkSpeed = 9600;    /* serial link speed */
 	common->wcmISDV4Speed = 19200;  /* serial ISDV4 link speed */
+	common->debugLevel = 0;         /* shared debug level can only 
+					 * be changed though xsetwacom */
 
 	common->wcmDevCls = &gWacomSerialDevice; /* device-specific functions */
 	common->wcmModel = NULL;                 /* model-specific functions */
@@ -204,9 +202,13 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 	common->wcmRotate = ROTATE_NONE; /* default tablet rotation to off */
 	common->wcmMaxCursorDist = 0;	/* Max distance received so far */
 	common->wcmCursorProxoutDist = 0;
-				/* Max mouse distance for proxy-out max/256 units */
+			/* Max mouse distance for proxy-out max/256 units */
 	common->wcmCursorProxoutDistDefault = PROXOUT_GRAPHIRE_DISTANCE; 
-				/* default to Graphire */
+			/* default to Graphire */
+	common->wcmSuppress = DEFAULT_SUPPRESS;    
+			/* transmit position if increment is superior */
+	common->wcmRawSample = DEFAULT_SAMPLES;    
+			/* number of raw data to be used to for filtering */
 
 	/* tool */
 	priv->tool = tool;
@@ -339,7 +341,7 @@ static void xf86WcmUninit(InputDriverPtr drv, LocalDevicePtr local, int flags)
 {
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
     
-	DBG(1, ErrorF("xf86WcmUninit\n"));
+	DBG(1, priv->debugLevel, ErrorF("xf86WcmUninit\n"));
 
 	gWacomModule.DevProc(local->dev, DEVICE_OFF);
 
@@ -364,8 +366,9 @@ static Bool xf86WcmMatchDevice(LocalDevicePtr pMatch, LocalDevicePtr pLocal)
 		(pMatch->device_control == gWacomModule.DevProc) &&
 		!strcmp(privMatch->common->wcmDevice, common->wcmDevice))
 	{
-		DBG(2, ErrorF("xf86WcmInit wacom port share between"
-				" %s and %s\n", pLocal->name, pMatch->name));
+		DBG(2, priv->debugLevel, ErrorF(
+			"xf86WcmInit wacom port share between"
+			" %s and %s\n", pLocal->name, pMatch->name));
 		type = xf86FindOptionValue(pMatch->options, "Type");
 		if ( type && (strstr(type, "eraser")) )
 			privMatch->common->wcmEraserID=pMatch->name;
@@ -377,14 +380,10 @@ static Bool xf86WcmMatchDevice(LocalDevicePtr pMatch, LocalDevicePtr pLocal)
 				privMatch->common->wcmEraserID=pLocal->name;
 			}
 		}
-		xfree(common->wcmDevices);
 		xfree(common);
 		common = priv->common = privMatch->common;
-		common->wcmNumDevices++;
-		common->wcmDevices = (LocalDevicePtr *)xrealloc(
-				common->wcmDevices,
-				sizeof(LocalDevicePtr) * common->wcmNumDevices);
-		common->wcmDevices[common->wcmNumDevices - 1] = pLocal;
+		priv->next = common->wcmDevices;
+		common->wcmDevices = priv;
 		return 1;
 	}
 	return 0;
@@ -405,7 +404,7 @@ static LocalDevicePtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 	WacomToolPtr tool = NULL;
 	WacomToolAreaPtr area = NULL;
 
-	gWacomModule.v4.wcmDrv = drv;
+	gWacomModule.wcmDrv = drv;
 
 	fakeLocal = (LocalDevicePtr) xcalloc(1, sizeof(LocalDeviceRec));
 	if (!fakeLocal)
@@ -482,11 +481,17 @@ static LocalDevicePtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 	xf86Msg(X_CONFIG, "%s device is %s\n", dev->identifier,
 			common->wcmDevice);
 
-	gWacomModule.debugLevel = xf86SetIntOption(local->options,
-		"DebugLevel", gWacomModule.debugLevel);
-	if (gWacomModule.debugLevel > 0)
-		xf86Msg(X_CONFIG, "WACOM: debug level set to %d\n",
-			gWacomModule.debugLevel);
+	priv->debugLevel = xf86SetIntOption(local->options,
+		"DebugLevel", priv->debugLevel);
+	if (priv->debugLevel > 0)
+		xf86Msg(X_CONFIG, "WACOM: %s debug level set to %d\n",
+			dev->identifier, priv->debugLevel);
+
+	common->debugLevel = xf86SetIntOption(local->options,
+		"CommonDBG", common->debugLevel);
+	if (common->debugLevel > 0)
+		xf86Msg(X_CONFIG, "WACOM: %s tablet common debug level set to %d\n",
+			dev->identifier, common->debugLevel);
 
 	s = xf86FindOptionValue(local->options, "Mode");
 
@@ -512,12 +517,11 @@ static LocalDevicePtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 	 * Always in absolute mode when it is not a core device.
 	 */
 	if (IsPad(priv))
-	{
-		if (local->flags & (XI86_ALWAYS_CORE | XI86_CORE_POINTER))
-			priv->flags &= ~ABSOLUTE_FLAG;
-		else
-			priv->flags |= ABSOLUTE_FLAG;
-	}
+		xf86WcmSetPadCoreMode(local);
+
+	/* Store original local Core flag so it can be changed later */
+	if (local->flags & (XI86_ALWAYS_CORE | XI86_CORE_POINTER))
+		priv->flags |= COREEVENT_FLAG;
 
 	xf86Msg(X_CONFIG, "%s is in %s mode\n", local->name,
 		(priv->flags & ABSOLUTE_FLAG) ? "absolute" : "relative");
@@ -549,10 +553,12 @@ static LocalDevicePtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 	common->wcmSuppress = xf86SetIntOption(local->options, "Suppress",
 			common->wcmSuppress);
 	if (common->wcmSuppress != 0) /* 0 disables suppression */
+	{
 		if (common->wcmSuppress > MAX_SUPPRESS)
 			common->wcmSuppress = MAX_SUPPRESS;
 		if (common->wcmSuppress < DEFAULT_SUPPRESS)
 			common->wcmSuppress = DEFAULT_SUPPRESS;
+	}
 	xf86Msg(X_CONFIG, "WACOM: suppress value is %d\n", common->wcmSuppress);      
     
 	if (xf86SetBoolOption(local->options, "Tilt",
@@ -840,33 +846,11 @@ static LocalDevicePtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 	s = xf86FindOptionValue(local->options, "Twinview");
 	if (s) xf86Msg(X_CONFIG, "%s: Twinview = %s\n", dev->identifier, s);
 	if (s && xf86NameCmp(s, "none") == 0) 
-	{
 		priv->twinview = TV_NONE;
-	}
 	else if (s && xf86NameCmp(s, "horizontal") == 0) 
-	{
 		priv->twinview = TV_LEFT_RIGHT;
-		/* default resolution */
-		if(!priv->tvResolution[0])
-		{
-			priv->tvResolution[0] = screenInfo.screens[0]->width/2;
-			priv->tvResolution[1] = screenInfo.screens[0]->height;
-			priv->tvResolution[2] = priv->tvResolution[0];
-			priv->tvResolution[3] = priv->tvResolution[1];
-		}
-	}
 	else if (s && xf86NameCmp(s, "vertical") == 0) 
-	{
 		priv->twinview = TV_ABOVE_BELOW;
-		/* default resolution */
-		if(!priv->tvResolution[0])
-		{
-			priv->tvResolution[0] = screenInfo.screens[0]->width;
-			priv->tvResolution[1] = screenInfo.screens[0]->height/2;
-			priv->tvResolution[2] = priv->tvResolution[0];
-			priv->tvResolution[3] = priv->tvResolution[1];
-		}
-	}
 	else if (s) 
 	{
 		xf86Msg(X_ERROR, "%s: invalid Twinview (should be none, vertical or horizontal). Using none.\n",
@@ -874,53 +858,6 @@ static LocalDevicePtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 		priv->twinview = TV_NONE;
 	}
 
-	/* initial screen info */
-	priv->screenTopX[0] = 0;
-	priv->screenTopY[0] = 0;
-	if (priv->twinview != TV_NONE)
-	{
-		priv->screenBottomX[0] = priv->tvResolution[0];
-		priv->screenBottomY[0] = priv->tvResolution[1];
-	}
-	else
-	{
-		priv->screenBottomX[0] = screenInfo.screens[0]->width;
-		priv->screenBottomY[0] = screenInfo.screens[0]->height;
-	}
-	if (priv->twinview != TV_NONE)
-	{
-		if (priv->twinview == TV_ABOVE_BELOW)
-		{
-			priv->screenTopX[1] = 0;
-			priv->screenTopY[1] = priv->tvResolution[1];
-			priv->screenBottomX[1] = priv->tvResolution[2];
-			priv->screenBottomY[1] = priv->tvResolution[3];
-		}
-		if (priv->twinview == TV_LEFT_RIGHT)
-		{
-			priv->screenTopX[1] = priv->tvResolution[0];
-			priv->screenTopY[1] = 0;
-			priv->screenBottomX[1] = priv->tvResolution[2];
-			priv->screenBottomY[1] = priv->tvResolution[3];
-		}
-	}
-	else if (screenInfo.numScreens)
-	{	
-		for (i=1; i<screenInfo.numScreens; i++)
-		{
-			int x = 0, y = 0, j;
-			for (j=0; j<i; j++)
-				x += screenInfo.screens[j]->width;
-			for (j=0; j<i; j++)
-				y += screenInfo.screens[j]->height;
-
-			priv->screenTopX[i] = x;
-			priv->screenTopY[i] = y;
-			priv->screenBottomX[i] = screenInfo.screens[i]->width;
-			priv->screenBottomY[i] = screenInfo.screens[i]->height;
-		}
-	}
-
 	/* mark the device configured */
 	local->flags |= XI86_POINTER_CAPABLE | XI86_CONFIGURED;
 
@@ -965,7 +902,7 @@ InputDriverRec WACOM =
 
 static void xf86WcmUnplug(pointer p)
 {
-	DBG(1, ErrorF("xf86WcmUnplug\n"));
+	ErrorF("xf86WcmUnplug\n");
 }
 
 /* xf86WcmPlug - called when the module subsection is found in XF86Config */
