@@ -1,25 +1,20 @@
 /*
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
  * Copyright 2002-2007 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
- *                                                                            
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is  hereby granted without fee, provided that
- * the  above copyright   notice appear  in   all  copies and  that both  that
- * copyright  notice   and   this  permission   notice  appear  in  supporting
- * documentation, and that   the  name of  Frederic   Lepied not  be  used  in
- * advertising or publicity pertaining to distribution of the software without
- * specific,  written      prior  permission.     Frederic  Lepied   makes  no
- * representations about the suitability of this software for any purpose.  It
- * is provided "as is" without express or implied warranty.                   
- *                                                                            
- * FREDERIC  LEPIED DISCLAIMS ALL   WARRANTIES WITH REGARD  TO  THIS SOFTWARE,
- * INCLUDING ALL IMPLIED   WARRANTIES OF MERCHANTABILITY  AND   FITNESS, IN NO
- * EVENT  SHALL FREDERIC  LEPIED BE   LIABLE   FOR ANY  SPECIAL, INDIRECT   OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA  OR PROFITS, WHETHER  IN  AN ACTION OF  CONTRACT,  NEGLIGENCE OR OTHER
- * TORTIOUS  ACTION, ARISING    OUT OF OR   IN  CONNECTION  WITH THE USE    OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software 
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #include "xf86Wacom.h"
@@ -48,11 +43,11 @@ static int serialEnableSuppressProtocol4(LocalDevicePtr local);
 static int serialSetLinkSpeedIntuos(LocalDevicePtr local);
 static int serialSetLinkSpeedProtocol5(LocalDevicePtr local);
 static int serialStartTablet(LocalDevicePtr local);
-static int serialParseCintiq(WacomCommonPtr common, const unsigned char* data);
-static int serialParseGraphire(WacomCommonPtr common, const unsigned char* data);
-static int serialParseProtocol4(WacomCommonPtr common, const unsigned char* data);
-static int serialParseProtocol5(WacomCommonPtr common, const unsigned char* data);
-static void serialParseP4Common(WacomCommonPtr common, const unsigned char* data, 
+static int serialParseCintiq(LocalDevicePtr local, const unsigned char* data);
+static int serialParseGraphire(LocalDevicePtr local, const unsigned char* data);
+static int serialParseProtocol4(LocalDevicePtr local, const unsigned char* data);
+static int serialParseProtocol5(LocalDevicePtr local, const unsigned char* data);
+static void serialParseP4Common(LocalDevicePtr local, const unsigned char* data, 
 	WacomDeviceState* last, WacomDeviceState* ds);
 
 /*****************************************************************************
@@ -112,7 +107,6 @@ static void serialParseP4Common(WacomCommonPtr common, const unsigned char* data
 		NULL,               /* link speed cannot be changed */
 		serialStartTablet,
 		serialParseCintiq,
-		xf86WcmHysteresisFilter,
 	};
 
 	static WacomModel serialPenPartner =
@@ -157,7 +151,6 @@ static void serialParseP4Common(WacomCommonPtr common, const unsigned char* data
 		NULL,               /* link speed cannot be changed */
 		serialStartTablet,
 		serialParseProtocol4,
-		xf86WcmHysteresisFilter,
 	};
 
 /*****************************************************************************
@@ -299,8 +292,6 @@ char* xf86WcmSendRequest(int fd, const char* request, char* answer, int maxlen)
 							strerror(errno));
 					return NULL;
 				}
-				DBG(10, ErrorF("%c err=%d [0]\n",
-					answer[0], nr));
 			}
 			maxtry--;  
 		} while ((answer[0] != request[0]) && maxtry);
@@ -329,8 +320,6 @@ char* xf86WcmSendRequest(int fd, const char* request, char* answer, int maxlen)
 								strerror(errno));
 						return NULL;
 					}
-					DBG(10, ErrorF("%c err=%d [1]\n",
-							answer[1], nr));
 				}
 				maxtry--;  
 			} while ((nr <= 0) && maxtry);
@@ -367,15 +356,10 @@ char* xf86WcmSendRequest(int fd, const char* request, char* answer, int maxlen)
 					ErrorF("Wacom xf86WcmRead error : %s\n", strerror(errno));
 					return NULL;
 				}
-				DBG(10, ErrorF("%c err=%d [%d]\n", answer[len], nr, len));
 			}
 			else
 			{
-				if (len == 2)
-				{
-					DBG(10, ErrorF("timeout remains %d tries\n", maxtry));
-					maxtry--;
-				}
+				if (len == 2) maxtry--;
 			}
 		} while ((nr <= 0) && len == 2 && maxtry);
 
@@ -410,8 +394,9 @@ static Bool serialDetect(LocalDevicePtr pDev)
 static Bool serialInit(LocalDevicePtr local)
 {
 	int err;
-    
-	DBG(1, ErrorF("initializing serial tablet\n"));    
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+
+	DBG(1, priv->debugLevel, ErrorF("initializing serial tablet\n"));    
 
 	/* Set the speed of the serial link to 38400 */
 	if (xf86WcmSetSerialSpeed(local->fd, 38400) < 0)
@@ -523,7 +508,8 @@ static int serialInitTablet(LocalDevicePtr local)
 	int loop, idx;
 	char id[BUFFER_SIZE];
 	float version;
-	WacomCommonPtr common =	((WacomDevicePtr)(local->private))->common;
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common =	priv->common;
 
 	WacomModelPtr model = NULL;
 
@@ -537,11 +523,11 @@ static int serialInitTablet(LocalDevicePtr local)
 	/* otherwise, query and initialize */
 	else
 	{
-		DBG(2, ErrorF("reading model\n"));
+		DBG(2, priv->debugLevel, ErrorF("reading model\n"));
 		if (!xf86WcmSendRequest(local->fd, WC_MODEL, id, sizeof(id)))
 			return !Success;
 
-		DBG(2, ErrorF("%s\n", id));
+		DBG(2, priv->debugLevel, ErrorF("%s\n", id));
   
 		if (xf86Verbose)
 			ErrorF("%s Wacom tablet model : %s\n",
@@ -595,9 +581,11 @@ static int serialInitTablet(LocalDevicePtr local)
 	return xf86WcmInitTablet(local,model,id,version);
 }
 
-static int serialParseGraphire(WacomCommonPtr common, const unsigned char* data)
+static int serialParseGraphire(LocalDevicePtr local, const unsigned char* data)
 {
 	int n;
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common = priv->common;
 	WacomDeviceState* last = &common->wcmChannel[0].valid.state;
 	WacomDeviceState* ds;
 
@@ -618,7 +606,7 @@ static int serialParseGraphire(WacomCommonPtr common, const unsigned char* data)
 	ds->buttons = (data[3] & BUTTONS_BITS) >> 3;
 	
 	/* requires button info, so it goes down here. */
-	serialParseP4Common(common,data,last,ds);
+	serialParseP4Common(local, data, last, ds);
 
 	/* handle relative wheel for non-stylus device */
 	if (ds->device_type == CURSOR_ID)
@@ -632,9 +620,11 @@ static int serialParseGraphire(WacomCommonPtr common, const unsigned char* data)
 	return common->wcmPktLength;
 }
 
-static int serialParseCintiq(WacomCommonPtr common, const unsigned char* data)
+static int serialParseCintiq(LocalDevicePtr local, const unsigned char* data)
 {
 	int n;
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common = priv->common;
 	WacomDeviceState* last = &common->wcmChannel[0].valid.state;
 	WacomDeviceState* ds;
 
@@ -666,18 +656,21 @@ static int serialParseCintiq(WacomCommonPtr common, const unsigned char* data)
 	ds->buttons = (data[3] & BUTTONS_BITS) >> 3;
 
 	/* requires button info, so it goes down here. */
-	serialParseP4Common(common,data,last,ds);
+	serialParseP4Common(local, data, last, ds);
 
 	xf86WcmEvent(common,0,ds);
 	return common->wcmPktLength;
 }
 
-static int serialParseProtocol4(WacomCommonPtr common, const unsigned char* data)
+static int serialParseProtocol4(LocalDevicePtr local, const unsigned char* data)
 {
 	int n;
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common = priv->common;
 	WacomDeviceState* last = &common->wcmChannel[0].valid.state;
 	WacomDeviceState* ds;
 
+	DBG(10, common->debugLevel, ErrorF("serialParseProtocol4 \n"));
 	/* positive value is skip */
 	if ((n = xf86WcmSerialValidate(common,data)) > 0) return n;
 
@@ -693,25 +686,28 @@ static int serialParseProtocol4(WacomCommonPtr common, const unsigned char* data
 
 	else
 		ds->pressure = (data[6] & ZAXIS_BITS) |
-			(data[6] & ZAXIS_SIGN_BIT) ? 0 : 0x40;
+			((data[6] & ZAXIS_SIGN_BIT) ? 0 : 0x40);
 
 	/* get button state */
 	ds->buttons = (data[3] & BUTTONS_BITS) >> 3;
 
 	/* requires button info, so it goes down here. */
-	serialParseP4Common(common,data,last,ds);
+	serialParseP4Common(local, data, last, ds);
 
 	xf86WcmEvent(common,0,ds);
 	return common->wcmPktLength;
 }
 
-static int serialParseProtocol5(WacomCommonPtr common, const unsigned char* data)
+static int serialParseProtocol5(LocalDevicePtr local, const unsigned char* data)
 {
 	int n;
 	int have_data=0;
 	int channel;
 	WacomDeviceState* ds;
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common = priv->common;
 
+	DBG(10, common->debugLevel, ErrorF("serialParseProtocol5 \n"));
 	/* positive value is skip */
 	if ((n = xf86WcmSerialValidate(common,data)) > 0) return n;
 
@@ -722,7 +718,7 @@ static int serialParseProtocol5(WacomCommonPtr common, const unsigned char* data
 	ds = &common->wcmChannel[channel].work;
 	RESET_RELATIVE(*ds);
 
-	DBG(7, ErrorF("packet header = %x\n", data[0]));
+	DBG(7, common->debugLevel, ErrorF("packet header = %x\n", data[0]));
      
 	/* Device ID packet */
 	if ((data[0] & 0xfc) == 0xc0)
@@ -750,7 +746,8 @@ static int serialParseProtocol5(WacomCommonPtr common, const unsigned char* data
 		else
 			ds->device_type = ERASER_ID;
   
-		DBG(6, ErrorF("device_id=%x serial_num=%u type=%s\n",
+		DBG(6, common->debugLevel, ErrorF(
+			"device_id=%x serial_num=%u type=%s\n",
 			ds->device_id, ds->serial_num,
 			(ds->device_type == STYLUS_ID) ? "stylus" :
 			(ds->device_type == CURSOR_ID) ? "cursor" :
@@ -859,7 +856,7 @@ static int serialParseProtocol5(WacomCommonPtr common, const unsigned char* data
 	}
 	else
 	{
-		DBG(10, ErrorF("unknown wacom V packet %x\n",data[0]));
+		DBG(10, common->debugLevel, ErrorF("unknown wacom V packet %x\n",data[0]));
 	}
 
 	/* if new data is available, send it */
@@ -876,8 +873,6 @@ static int serialParseProtocol5(WacomCommonPtr common, const unsigned char* data
 
 static void serialInitIntuos(WacomCommonPtr common, const char* id, float version)
 {
-	DBG(2, ErrorF("detected an Intuos model\n"));
-
 	common->wcmProtocolLevel = 5;
 	common->wcmVersion = version;
 
@@ -890,8 +885,6 @@ static void serialInitIntuos(WacomCommonPtr common, const char* id, float versio
 
 static void serialInitIntuos2(WacomCommonPtr common, const char* id, float version)
 {
-	DBG(2, ErrorF("detected an Intuos2 model\n"));
-
 	common->wcmProtocolLevel = 5;
 	common->wcmVersion = version;
 
@@ -904,8 +897,6 @@ static void serialInitIntuos2(WacomCommonPtr common, const char* id, float versi
 
 static void serialInitCintiq(WacomCommonPtr common, const char* id, float version)
 {
-	DBG(2, ErrorF("detected a Cintiq model\n"));
-
 	common->wcmProtocolLevel = 4;
 	common->wcmPktLength = 7;
 	common->wcmVersion = version;
@@ -977,8 +968,6 @@ static void serialInitCintiq(WacomCommonPtr common, const char* id, float versio
 
 static void serialInitPenPartner(WacomCommonPtr common, const char* id, float version)
 {
-	DBG(2, ErrorF("detected a PenPartner model\n"));
-
 	common->wcmProtocolLevel = 4;
 	common->wcmPktLength = 7;
 	common->wcmVersion = version;
@@ -990,8 +979,6 @@ static void serialInitPenPartner(WacomCommonPtr common, const char* id, float ve
 
 static void serialInitGraphire(WacomCommonPtr common, const char* id, float version)
 {
-	DBG(2, ErrorF("detected a Graphire model\n"));
-
 	common->wcmProtocolLevel = 4;
 	common->wcmPktLength = 7;
 	common->wcmVersion = version;
@@ -1006,8 +993,6 @@ static void serialInitGraphire(WacomCommonPtr common, const char* id, float vers
 
 static void serialInitProtocol4(WacomCommonPtr common, const char* id, float version)
 {
-	DBG(2, ErrorF("detected a Protocol4 model\n"));
-
 	common->wcmProtocolLevel = 4;
 	common->wcmPktLength = 7;
 	common->wcmVersion = version;
@@ -1027,15 +1012,17 @@ static void serialGetResolution(LocalDevicePtr local)
 {
 	int a, b;
 	char buffer[BUFFER_SIZE], header[BUFFER_SIZE];
-	WacomCommonPtr common =	((WacomDevicePtr)(local->private))->common;
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common =	priv->common;
 
 	if (!(common->wcmResolX && common->wcmResolY))
 	{
-		DBG(2, ErrorF("Requesting resolution from device\n"));
+		DBG(2, priv->debugLevel, ErrorF(
+			"Requesting resolution from device\n"));
 		if (xf86WcmSendRequest(local->fd, WC_CONFIG, buffer,
 			sizeof(buffer)))
 		{
-			DBG(2, ErrorF("%s\n", buffer));
+			DBG(2, priv->debugLevel, ErrorF("%s\n", buffer));
 			/* The header string is simply a place to put the
 			 * unwanted config header don't use buffer+xx because
 			 * the header size varies on different tablets */
@@ -1044,8 +1031,8 @@ static void serialGetResolution(LocalDevicePtr local)
 				 &a, &b, &common->wcmResolX,
 				 &common->wcmResolY) == 5)
 			{
-				DBG(6, ErrorF("WC_CONFIG Header = %s\n",
-					header));
+				DBG(6, priv->debugLevel, ErrorF(
+					"WC_CONFIG Header = %s\n", header));
 			}
 			else
 			{
@@ -1062,18 +1049,19 @@ static void serialGetResolution(LocalDevicePtr local)
 		}
 	}
 
-	DBG(2, ErrorF("serialGetResolution: ResolX=%d ResolY=%d\n",
+	DBG(2, priv->debugLevel, ErrorF("serialGetResolution: ResolX=%d ResolY=%d\n",
 		common->wcmResolX, common->wcmResolY));
 }
 
 static int serialGetRanges(LocalDevicePtr local)
 {
 	char buffer[BUFFER_SIZE];
-	WacomCommonPtr common =	((WacomDevicePtr)(local->private))->common;
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common =	priv->common;
 
 	if (!(common->wcmMaxX && common->wcmMaxY))
 	{
-		DBG(2, ErrorF("Requesting max coordinates\n"));
+		DBG(2, priv->debugLevel, ErrorF("Requesting max coordinates\n"));
 		if (!xf86WcmSendRequest(local->fd, WC_COORD, buffer,
 			sizeof(buffer)))
 		{
@@ -1081,7 +1069,7 @@ static int serialGetRanges(LocalDevicePtr local)
 				"Use the MaxX and MaxY options.\n");
 			return !Success;
 		}
-		DBG(2, ErrorF("%s\n", buffer));
+		DBG(2, priv->debugLevel, ErrorF("%s\n", buffer));
 		if (sscanf(buffer+2, "%d,%d", &common->wcmMaxX,
 			&common->wcmMaxY) != 2)
 		{
@@ -1091,7 +1079,7 @@ static int serialGetRanges(LocalDevicePtr local)
 		}
 	}
 
-	DBG(2, ErrorF("serialGetRanges: maxX=%d maxY=%d (%g,%g in)\n",
+	DBG(2, priv->debugLevel, ErrorF("serialGetRanges: maxX=%d maxY=%d (%g,%g in)\n",
 		common->wcmMaxX, common->wcmMaxY,
 		(double)common->wcmMaxX / common->wcmResolX,
 		(double)common->wcmMaxY / common->wcmResolY));
@@ -1192,9 +1180,10 @@ static int serialSetLinkSpeedProtocol5(LocalDevicePtr local)
 {
 	int err;
 	char* speed_init_string;
-	WacomCommonPtr common =	((WacomDevicePtr)(local->private))->common;
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common =	priv->common;
 
-	DBG(1, ErrorF("Switching serial link to %d\n",
+	DBG(1, priv->debugLevel, ErrorF("Switching serial link to %d\n",
 		common->wcmLinkSpeed));
 
 	/* set init string according to speed */
@@ -1238,10 +1227,12 @@ static int serialStartTablet(LocalDevicePtr local)
 	return Success;
 }
 
-static void serialParseP4Common(WacomCommonPtr common,
+static void serialParseP4Common(LocalDevicePtr local,
 	const unsigned char* data, WacomDeviceState* last,
 	WacomDeviceState* ds)
 {
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common = priv->common;
 	int is_stylus = (data[0] & POINTER_BIT);
 	int cur_type = is_stylus ?
 		((ds->buttons & 4) ? ERASER_ID : STYLUS_ID) :
@@ -1281,7 +1272,7 @@ static void serialParseP4Common(WacomCommonPtr common,
 		{
 			/* send a prox-out for old device */
 			WacomDeviceState out = { 0 };
-			xf86WcmEvent(common,0,&out);
+			xf86WcmEvent(common, 0, &out);
 			ds->device_type = cur_type;
 		}
 	}
@@ -1296,7 +1287,7 @@ static void serialParseP4Common(WacomCommonPtr common,
 		ds->device_id = ERASER_DEVICE_ID;
 	}
 		
-	DBG(8, ErrorF("serialParseP4Common %s\n",
+	DBG(8, common->debugLevel, ErrorF("serialParseP4Common %s\n",
 		ds->device_type == CURSOR_ID ? "CURSOR" :
 		ds->device_type == ERASER_ID ? "ERASER " :
 		ds->device_type == STYLUS_ID ? "STYLUS" : "NONE"));
@@ -1318,8 +1309,8 @@ int xf86WcmSerialValidate(WacomCommonPtr common, const unsigned char* data)
 				((i!=0) && (data[i] & HEADER_BIT)) )
 		{
 			bad = 1;
-			DBG(10, ErrorF("xf86WcmSerialValidate: bad magic at %d "
-				"v=%x l=%d\n", i, data[i], common->wcmPktLength));
+			ErrorF("xf86WcmSerialValidate: bad magic at %d "
+				"v=%x l=%d\n", i, data[i], common->wcmPktLength);
 			if (i!=0 && (data[i] & HEADER_BIT)) return i;
 		}
 	}
