@@ -2,24 +2,19 @@
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
  * Copyright 2002-2007 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
  *                                                                            
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is  hereby granted without fee, provided that
- * the  above copyright   notice appear  in   all  copies and  that both  that
- * copyright  notice   and   this  permission   notice  appear  in  supporting
- * documentation, and that   the  name of  Frederic   Lepied not  be  used  in
- * advertising or publicity pertaining to distribution of the software without
- * specific,  written      prior  permission.     Frederic  Lepied   makes  no
- * representations about the suitability of this software for any purpose.  It
- * is provided "as is" without express or implied warranty.                   
- *                                                                            
- * FREDERIC  LEPIED DISCLAIMS ALL   WARRANTIES WITH REGARD  TO  THIS SOFTWARE,
- * INCLUDING ALL IMPLIED   WARRANTIES OF MERCHANTABILITY  AND   FITNESS, IN NO
- * EVENT  SHALL FREDERIC  LEPIED BE   LIABLE   FOR ANY  SPECIAL, INDIRECT   OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA  OR PROFITS, WHETHER  IN  AN ACTION OF  CONTRACT,  NEGLIGENCE OR OTHER
- * TORTIOUS  ACTION, ARISING    OUT OF OR   IN  CONNECTION  WITH THE USE    OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software 
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #include "xf86Wacom.h"
@@ -58,11 +53,11 @@ static void usbInitProtocol5(WacomCommonPtr common, const char* id,
 static void usbInitProtocol4(WacomCommonPtr common, const char* id,
 	float version);
 int usbWcmGetRanges(LocalDevicePtr local);
-static int usbParse(WacomCommonPtr common, const unsigned char* data);
+static int usbParse(LocalDevicePtr local, const unsigned char* data);
 static int usbDetectConfig(LocalDevicePtr local);
-static void usbParseEvent(WacomCommonPtr common,
+static void usbParseEvent(LocalDevicePtr local,
 	const struct input_event* event);
-static void usbParseChannel(WacomCommonPtr common, int channel, int serial);
+static void usbParseChannel(LocalDevicePtr local, int channel, int serial);
 
 	WacomDeviceClass gWacomUSBDevice =
 	{
@@ -167,6 +162,54 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial);
 		usbDetectConfig,      /* detect hardware buttons etc */
 	};
 
+	static WacomModel usbBamboo =
+	{
+		"USB Bamboo",
+		usbInitProtocol4,
+		NULL,                 /* resolution not queried */
+		usbWcmGetRanges,
+		NULL,                 /* reset not supported */
+		NULL,                 /* tilt automatically enabled */
+		NULL,                 /* suppress implemented in software */
+		NULL,                 /* link speed unsupported */
+		NULL,                 /* start not supported */
+		usbParse,
+		xf86WcmFilterCoord,   /* input filtering */
+		usbDetectConfig,      /* detect hardware buttons etc */
+	};
+
+	static WacomModel usbBamboo1 =
+	{
+		"USB Bamboo1",
+		usbInitProtocol4,
+		NULL,                 /* resolution not queried */
+		usbWcmGetRanges,
+		NULL,                 /* reset not supported */
+		NULL,                 /* tilt automatically enabled */
+		NULL,                 /* suppress implemented in software */
+		NULL,                 /* link speed unsupported */
+		NULL,                 /* start not supported */
+		usbParse,
+		xf86WcmFilterCoord,   /* input filtering */
+		usbDetectConfig,      /* detect hardware buttons etc */
+	};
+
+	static WacomModel usbBambooFun =
+	{
+		"USB BambooFun",
+		usbInitProtocol4,
+		NULL,                 /* resolution not queried */
+		usbWcmGetRanges,
+		NULL,                 /* reset not supported */
+		NULL,                 /* tilt automatically enabled */
+		NULL,                 /* suppress implemented in software */
+		NULL,                 /* link speed unsupported */
+		NULL,                 /* start not supported */
+		usbParse,
+		xf86WcmFilterCoord,   /* input filtering */
+		usbDetectConfig,      /* detect hardware buttons etc */
+	};
+
 	static WacomModel usbCintiq =
 	{
 		"USB Cintiq",
@@ -179,7 +222,7 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial);
 		NULL,                 /* link speed unsupported */
 		NULL,                 /* start not supported */
 		usbParse,
-		xf86WcmHysteresisFilter,   /* input filtering */
+		NULL,                 /* input filtering */
 		usbDetectConfig,      /* detect hardware buttons etc */
 	};
 
@@ -195,7 +238,7 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial);
 		NULL,                 /* link speed unsupported */
 		NULL,                 /* start not supported */
 		usbParse,
-		xf86WcmHysteresisFilter,   /* input filtering */
+		NULL,                 /* input filtering */
 		usbDetectConfig,      /* detect hardware buttons etc */
 	};
 
@@ -281,7 +324,7 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial);
 
 	static WacomModel usbCintiqV5 =
 	{
-		"USB Cintiq21UX",
+		"USB CintiqV5",
 		usbInitProtocol5,
 		NULL,                 /* resolution not queried */
 		usbWcmGetRanges,
@@ -304,14 +347,17 @@ static Bool usbDetect(LocalDevicePtr local)
 {
 	int version;
 	int err;
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 
-	DBG(1, ErrorF("usbDetect\n"));
+	DBG(1, priv->debugLevel, ErrorF("usbDetect\n"));
     
 	SYSCALL(err = ioctl(local->fd, EVIOCGVERSION, &version));
 
 	if (err < 0)
+	{
+		ErrorF("usbDetect: can not ioctl version\n");
 		return 0;
-
+	}
 #ifdef EVIOCGRAB
 	/* Try to grab the event device so that data don't leak to /dev/input/mice */
 	SYSCALL(err = ioctl(local->fd, EVIOCGRAB, (pointer)1));
@@ -363,6 +409,8 @@ static struct
 	{ 0x14, 2032, 2032, &usbGraphire3  }, /* Graphire3 6x8 */
 	{ 0x15, 2032, 2032, &usbGraphire4  }, /* Graphire4 4x5 */
 	{ 0x16, 2032, 2032, &usbGraphire4  }, /* Graphire4 6x8 */ 
+	{ 0x17, 2540, 2540, &usbBambooFun  }, /* BambooFun 4x5 */
+	{ 0x18, 2540, 2540, &usbBambooFun  }, /* BambooFun 6x8 */
 	{ 0x81, 2032, 2032, &usbGraphire4  }, /* Graphire4 6x8 BlueTooth */
 
 	{ 0x20, 2540, 2540, &usbIntuos     }, /* Intuos 4x5 */
@@ -383,7 +431,7 @@ static struct
 	{ 0x38,  508,  508, &usbCintiq     }, /* PL510 */
 	{ 0x39,  508,  508, &usbCintiq     }, /* PL710 */ 
 	{ 0xC0,  508,  508, &usbCintiq     }, /* DTF720 */
-	{ 0xC4,  508,  508, &usbCintiq     }, /* DTF520 */ 
+	{ 0xC4,  508,  508, &usbCintiq     }, /* DTF521 */ 
 
 	{ 0x41, 2540, 2540, &usbIntuos2    }, /* Intuos2 4x5 */
 	{ 0x42, 2540, 2540, &usbIntuos2    }, /* Intuos2 6x8 */
@@ -399,6 +447,9 @@ static struct
 	{ 0x63, 1016, 1016, &usbVolito2    }, /* Volito2 2x3 */
 	{ 0x64, 1016, 1016, &usbVolito2    }, /* PenPartner2 */
 
+	{ 0x65, 2540, 2540, &usbBamboo     }, /* Bamboo */
+	{ 0x69, 1012, 1012, &usbBamboo1    }, /* Bamboo1 */ 
+
 	{ 0xB0, 5080, 5080, &usbIntuos3    }, /* Intuos3 4x5 */
 	{ 0xB1, 5080, 5080, &usbIntuos3    }, /* Intuos3 6x8 */
 	{ 0xB2, 5080, 5080, &usbIntuos3    }, /* Intuos3 9x12 */
@@ -407,7 +458,8 @@ static struct
 	{ 0xB5, 5080, 5080, &usbIntuos3    }, /* Intuos3 6x11 */
 	{ 0xB7, 5080, 5080, &usbIntuos3    }, /* Intuos3 4x6 */
 
-	{ 0x3F, 5080, 5080, &usbCintiqV5   }  /* Cintiq 21UX */ 
+	{ 0x3F, 5080, 5080, &usbCintiqV5   }, /* Cintiq 21UX */ 
+	{ 0xC6, 5080, 5080, &usbCintiqV5   }  /* Cintiq 12WX */ 
 };
 
 Bool usbWcmInit(LocalDevicePtr local)
@@ -420,7 +472,7 @@ Bool usbWcmInit(LocalDevicePtr local)
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomCommonPtr common = priv->common;
 
-	DBG(1, ErrorF("initializing USB tablet\n"));    
+	DBG(1, priv->debugLevel, ErrorF("initializing USB tablet\n"));    
 
 	/* fetch vendor, product, and model name */
 	ioctl(local->fd, EVIOCGID, sID);
@@ -481,7 +533,6 @@ Bool usbWcmInit(LocalDevicePtr local)
 static void usbInitProtocol5(WacomCommonPtr common, const char* id,
 	float version)
 {
-	DBG(2, ErrorF("detected a protocol 5 model (%s)\n",id));
 	common->wcmProtocolLevel = 5;
 	common->wcmPktLength = sizeof(struct input_event);
 	common->wcmCursorProxoutDistDefault 
@@ -494,7 +545,6 @@ static void usbInitProtocol5(WacomCommonPtr common, const char* id,
 static void usbInitProtocol4(WacomCommonPtr common, const char* id,
 	float version)
 {
-	DBG(2, ErrorF("detected a protocol 4 model (%s)\n",id));
 	common->wcmProtocolLevel = 4;
 	common->wcmPktLength = sizeof(struct input_event);
 	common->wcmCursorProxoutDistDefault 
@@ -518,12 +568,11 @@ int usbWcmGetRanges(LocalDevicePtr local)
 	   to indicate end of record for a complete tablet event */
 	if (ISBITSET(ev,EV_SYN))
 	{
-		DBG(2, ErrorF("WACOM: Kernel supports SYN_REPORTs\n"));
 		common->wcmFlags |= USE_SYN_REPORTS_FLAG;
 	}
 	else
 	{
-		DBG(2, ErrorF("WACOM: Kernel doesn't support SYN_REPORTs\n"));
+		ErrorF("WACOM: Kernel doesn't support SYN_REPORT\n");
 		common->wcmFlags &= ~USE_SYN_REPORTS_FLAG;
 	}
 
@@ -606,7 +655,7 @@ static int usbDetectConfig(LocalDevicePtr local)
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomCommonPtr common = priv->common;
 
-	DBG(10, ErrorF("usbDetectConfig \n"));
+	DBG(10, common->debugLevel, ErrorF("usbDetectConfig \n"));
 	if (IsPad (priv))
 	{
 		priv->nbuttons = common->npadkeys;
@@ -637,17 +686,23 @@ static int usbDetectConfig(LocalDevicePtr local)
 	return TRUE;
 }
 
-static int usbParse(WacomCommonPtr common, const unsigned char* data)
+static int usbParse(LocalDevicePtr local, const unsigned char* data)
 {
-	usbParseEvent(common,(const struct input_event*)data);
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common = priv->common;
+
+	usbParseEvent(local, (const struct input_event*)data);
 	return common->wcmPktLength;
 }
 
-static void usbParseEvent(WacomCommonPtr common,
+static void usbParseEvent(LocalDevicePtr local,
 	const struct input_event* event)
 {
 	int i, channel;
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common = priv->common;
 
+	DBG(10, common->debugLevel, ErrorF("usbParseEvent \n"));
 	/* store events until we receive the MSC_SERIAL containing
 	 * the serial number; without it we cannot determine the
 	 * correct channel. */
@@ -656,8 +711,8 @@ static void usbParseEvent(WacomCommonPtr common,
 	if (common->wcmEventCnt >=
 		(sizeof(common->wcmEvents)/sizeof(*common->wcmEvents)))
 	{
-		DBG(1, ErrorF("usbParse: Exceeded event queue (%d)\n",
-				common->wcmEventCnt));
+		ErrorF("usbParse: Exceeded event queue (%d)\n",
+				common->wcmEventCnt);
 		common->wcmEventCnt = 0;
 		common->wcmLastToolSerial = 0;
 		return;
@@ -685,7 +740,7 @@ static void usbParseEvent(WacomCommonPtr common,
 		   using SYN_REPORT as the end of record indicator */
 		if (! USE_SYN_REPORTS(common))
 		{
-			DBG(2, ErrorF("WACOM: Got unexpected SYN_REPORT, changing mode\n"));
+			ErrorF("WACOM: Got unexpected SYN_REPORT, changing mode\n");
 
 			/* we can expect SYN_REPORT's from now on */
 			common->wcmFlags |= USE_SYN_REPORTS_FLAG;
@@ -701,59 +756,73 @@ static void usbParseEvent(WacomCommonPtr common,
 
 	/* figure out the channel to use based on serial number */
 	channel = -1;
-
-	/* find existing channel */
-	for (i=0; i<MAX_CHANNELS; ++i)
+	if (common->wcmProtocolLevel == 4)
 	{
-		if (common->wcmChannel[i].work.proximity &&
-		   common->wcmChannel[i].work.serial_num == common->wcmLastToolSerial)
+		/* Protocol 4 don't support tool serial numbers */
+		if (common->wcmLastToolSerial == 0xf0)
+			channel = 1;
+		else
+			channel = 0;
+		if (!common->wcmChannel[channel].work.proximity)
 		{
-			channel = i;
-			break;
+			memset(&common->wcmChannel[channel],0,sizeof(WacomChannel));
+			/* in case the in-prox event was missing */
+			common->wcmChannel[channel].work.proximity = 1;
 		}
 	}
-
-	/* find an empty channel */
-	if (channel < 0)
+	else
 	{
+		/* find existing channel */
 		for (i=0; i<MAX_CHANNELS; ++i)
 		{
-			if (!common->wcmChannel[i].work.proximity)
+			if (common->wcmChannel[i].work.proximity &&
+			   common->wcmChannel[i].work.serial_num == common->wcmLastToolSerial)
 			{
-				memset(&common->wcmChannel[i],0,
-						sizeof(WacomChannel));
-				/* in case the in-prox event was missing */
-				common->wcmChannel[i].work.proximity = 1;
-				/* Intuos3 or Graphire4 Pad */
-				if (common->wcmLastToolSerial == 0xffffffff ||
-					common->wcmLastToolSerial == 0xf0)
-					common->wcmChannel[i].work.device_type = PAD_ID;
 				channel = i;
 				break;
 			}
 		}
+
+		/* find an empty channel */
+		if (channel < 0)
+		{
+			for (i=0; i<MAX_CHANNELS; ++i)
+			{
+				if (!common->wcmChannel[i].work.proximity)
+				{
+					memset(&common->wcmChannel[i],0,
+							sizeof(WacomChannel));
+					/* in case the in-prox event was missing */
+					common->wcmChannel[i].work.proximity = 1;
+					channel = i;
+					break;
+				}
+			}
+		}
 	}
 
 	/* fresh out of channels */
 	if (channel < 0)
 	{
 		/* this should never happen in normal use */
-		DBG(1, ErrorF("usbParse: Exceeded channel count; "
-			"ignoring.\n"));
+		ErrorF("usbParse: Exceeded channel count; "
+			"ignoring.\n");
 		return;
 
 	}
 
-	usbParseChannel(common,channel,common->wcmLastToolSerial);
+	usbParseChannel(local,channel,common->wcmLastToolSerial);
 	common->wcmLastToolSerial = 0;
 	common->wcmEventCnt = 0;
 }
 
-static void usbParseChannel(WacomCommonPtr common, int channel, int serial)
+static void usbParseChannel(LocalDevicePtr local, int channel, int serial)
 {
 	int i, shift, nkeys;
 	WacomDeviceState* ds;
 	struct input_event* event;
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common = priv->common;
 
 	#define MOD_BUTTONS(bit, value) do { \
 		shift = 1<<bit; \
@@ -761,6 +830,7 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial)
 		(ds->buttons | (shift)) : (ds->buttons & ~(shift))); \
 		} while (0)
 
+	DBG(6, common->debugLevel, ErrorF("usbParseChannel \n"));
 	/* all USB data operates from previous context except relative values*/
 	ds = &common->wcmChannel[channel].work;
 	ds->relwheel = 0;
@@ -770,9 +840,9 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial)
 	for (i=0; i<common->wcmEventCnt; ++i)
 	{
 		event = common->wcmEvents + i;
-		DBG(11, ErrorF("usbParseChannel event[%d]->type=%d "
-			"code=%d value=%d\n", i, event->type,
-			event->code, event->value));
+		DBG(11, common->debugLevel, ErrorF("usbParseChannel "
+			"event[%d]->type=%d code=%d value=%d\n", 
+			i, event->type, event->code, event->value));
 
 		/* absolute events */
 		if (event->type == EV_ABS)
@@ -785,7 +855,7 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial)
 				ds->stripx = event->value; 
 			else if (event->code == ABS_RY)
 				ds->stripy = event->value;
-			else if (event->code == ABS_RZ)
+			else if (event->code == ABS_RZ) 
 				ds->rotation = event->value;
 			else if (event->code == ABS_TILT_X)
 				ds->tiltx = event->value - 64;
@@ -795,7 +865,8 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial)
 				ds->pressure = event->value;
 			else if (event->code == ABS_DISTANCE)
 				ds->distance = event->value;
-			else if (event->code == ABS_WHEEL)
+			else if (event->code == ABS_WHEEL || 
+				    event->code == ABS_Z)
 				ds->abswheel = event->value;
 			else if (event->code == ABS_THROTTLE)
 				ds->throttle = event->value;
@@ -819,7 +890,8 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial)
 			{
 				ds->device_type = STYLUS_ID;
 				ds->proximity = (event->value != 0);
-				DBG(6, ErrorF("USB stylus detected %x\n",
+				DBG(6, common->debugLevel, ErrorF(
+					"USB stylus detected %x\n",
 					event->code));
 			}
 			else if (event->code == BTN_TOOL_RUBBER)
@@ -828,20 +900,23 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial)
 				ds->proximity = (event->value != 0);
 				if (ds->proximity)
 					ds->proximity = ERASER_PROX;
-				DBG(6, ErrorF("USB eraser detected %x\n",
+				DBG(6, common->debugLevel, ErrorF(
+					"USB eraser detected %x\n",
 					event->code));
 			}
 			else if ((event->code == BTN_TOOL_MOUSE) ||
 				(event->code == BTN_TOOL_LENS))
 			{
-				DBG(6, ErrorF("USB mouse detected %x\n",
+				DBG(6, common->debugLevel, ErrorF(
+					"USB mouse detected %x\n",
 					event->code));
 				ds->device_type = CURSOR_ID;
 				ds->proximity = (event->value != 0);
 			}
 			else if (event->code == BTN_TOOL_FINGER)
 			{
-				DBG(6, ErrorF("USB Pad detected %x\n",
+				DBG(6, common->debugLevel, ErrorF(
+					"USB Pad detected %x\n",
 					event->code));
 				ds->device_type = PAD_ID;
 				ds->proximity = (event->value != 0);
@@ -878,10 +953,15 @@ static void usbParseChannel(WacomCommonPtr common, int channel, int serial)
 		}
 	} /* next event */
 
+	/* it is an out-prox when id or/and serial number is zero for kernel 2.4 */
+	if ((!ds->device_id || !ds->serial_num) && !USE_SYN_REPORTS(common))
+ 		ds->proximity = 0;
+
 	/* DTF720 doesn't support eraser */
 	if (common->tablet_id == 0xC0 && ds->device_type == ERASER_ID) 
 	{
-		DBG(10, ErrorF("usbParseChannel DTF720 doesn't support eraser "));
+		DBG(10, common->debugLevel, ErrorF("usbParseChannel "
+			"DTF 720 doesn't support eraser "));
 		return;
 	}
 
