@@ -2,24 +2,19 @@
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
  * Copyright 2002-2007 by Ping Cheng, Wacom Technology. <pingc@wacom.com>
  * 
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is  hereby granted without fee, provided that
- * the  above copyright   notice appear  in   all  copies and  that both  that
- * copyright  notice   and   this  permission   notice  appear  in  supporting
- * documentation, and that   the  name of  Frederic   Lepied not  be  used  in
- * advertising or publicity pertaining to distribution of the software without
- * specific,  written      prior  permission.     Frederic  Lepied   makes  no
- * representations about the suitability of this software for any purpose.  It
- * is provided "as is" without express or implied warranty.
- * 
- * FREDERIC  LEPIED DISCLAIMS ALL   WARRANTIES WITH REGARD  TO  THIS SOFTWARE,
- * INCLUDING ALL IMPLIED   WARRANTIES OF MERCHANTABILITY  AND   FITNESS, IN NO
- * EVENT  SHALL FREDERIC  LEPIED BE   LIABLE   FOR ANY  SPECIAL, INDIRECT   OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA  OR PROFITS, WHETHER  IN  AN ACTION OF  CONTRACT,  NEGLIGENCE OR OTHER
- * TORTIOUS  ACTION, ARISING    OUT OF OR   IN  CONNECTION  WITH THE USE    OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software 
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #ifndef __XF86_XF86WACOM_H
@@ -100,9 +95,9 @@
 
 #define DEBUG 1
 #if DEBUG
-#define DBG(lvl, f) do { if ((lvl) <= gWacomModule.debugLevel) f; } while (0)
+#define DBG(lvl, dLevel, f) do { if ((lvl) <= dLevel) f; } while (0)
 #else
-#define DBG(lvl, f)
+#define DBG(lvl, dLevel, f)
 #endif
 
 /*****************************************************************************
@@ -118,7 +113,7 @@
 #define DEFAULT_SPEED 1.0       /* default relative cursor speed */
 #define MAX_ACCEL 7             /* number of acceleration levels */
 #define DEFAULT_SUPPRESS 2      /* default suppress */
-#define MAX_SUPPRESS 20         /* max value of suppress */
+#define MAX_SUPPRESS 100        /* max value of suppress */
 #define BUFFER_SIZE 256         /* size of reception buffer */
 #define XI_STYLUS "STYLUS"      /* X device name for the stylus */
 #define XI_CURSOR "CURSOR"      /* X device name for the cursor */
@@ -154,8 +149,6 @@
  *****************************************************************************/
 
 typedef struct _WacomModule WacomModule;
-typedef struct _WacomModule4 WacomModule4;
-typedef struct _WacomModule3 WacomModule3;
 typedef struct _WacomModel WacomModel, *WacomModelPtr;
 typedef struct _WacomDeviceRec WacomDeviceRec, *WacomDevicePtr;
 typedef struct _WacomDeviceState WacomDeviceState, *WacomDeviceStatePtr;
@@ -171,17 +164,11 @@ typedef struct _WacomToolArea WacomToolArea, *WacomToolAreaPtr;
  *               global namespaces as clean as possible.
  *****************************************************************************/
 
-struct _WacomModule4
-{
-	InputDriverPtr wcmDrv;
-};
-
 struct _WacomModule
 {
-	int debugLevel;
 	const char* identification;
 
-	WacomModule4 v4;
+	InputDriverPtr wcmDrv;
 
 	int (*DevOpen)(DeviceIntPtr pWcm);
 	void (*DevReadInput)(LocalDevicePtr local);
@@ -214,7 +201,7 @@ struct _WacomModel
 	int (*EnableSuppress)(LocalDevicePtr local);
 	int (*SetLinkSpeed)(LocalDevicePtr local);
 	int (*Start)(LocalDevicePtr local);
-	int (*Parse)(WacomCommonPtr common, const unsigned char* data);
+	int (*Parse)(LocalDevicePtr local, const unsigned char* data);
 	int (*FilterRaw)(WacomCommonPtr common, WacomChannelPtr pChannel,
 		WacomDeviceStatePtr ds);
 	int (*DetectConfig)(LocalDevicePtr local);
@@ -240,6 +227,7 @@ struct _WacomModel
 #define BUTTONS_ONLY_FLAG	0x00000080
 #define TPCBUTTONS_FLAG		0x00000100
 #define TPCBUTTONONE_FLAG	0x00000200
+#define COREEVENT_FLAG		0x00000400
 
 #define IsCursor(priv) (DEVICE_ID((priv)->flags) == CURSOR_ID)
 #define IsStylus(priv) (DEVICE_ID((priv)->flags) == STYLUS_ID)
@@ -261,11 +249,13 @@ typedef int (*FILTERFUNC)(WacomDevicePtr pDev, WacomDeviceStatePtr pState);
 					 * tablet buttons besides the strips are
 					 * treated as buttons */
 
-typedef enum { TV_NONE = 0, TV_ABOVE_BELOW = 1, TV_LEFT_RIGHT = 2 } tvMode;
-
 struct _WacomDeviceRec
 {
 	/* configuration fields */
+	struct _WacomDeviceRec *next;
+	LocalDevicePtr local;
+	int debugLevel;
+
 	unsigned int flags;     /* various flags (type, abs, touch...) */
 	int topX;               /* X top */
 	int topY;               /* Y top */
@@ -279,7 +269,7 @@ struct _WacomDeviceRec
 	int screenTopY[32];	/* top cordinate of the associated screen */
 	int screenBottomX[32];	/* right cordinate of the associated screen */
 	int screenBottomY[32];	/* bottom cordinate of the associated screen */
- 	int button[MAX_BUTTONS];/* buttons assignments */
+	int button[MAX_BUTTONS];/* buttons assignments */
 	unsigned keys[MAX_BUTTONS][256]; /* keystrokes assigned to buttons */
 	int relup;
 	unsigned rupk[256]; /* keystrokes assigned to relative wheel up event (default is button 4) */
@@ -297,10 +287,10 @@ struct _WacomDeviceRec
 	unsigned srupk[256]; /* keystrokes assigned to right strip up event (default is button 4) */
 	int striprdn;
  	unsigned srdnk[256]; /* keystrokes assigned to right strip up event (default is button 4) */
-	int nbuttons;           /* number of buttons for this subdevice */
-	int naxes;              /* number of axes */
-  
-	WacomCommonPtr common;  /* common info pointer */
+	int nbuttons;         /* number of buttons for this subdevice */
+	int naxes;            /* number of axes */
+
+	WacomCommonPtr common;/* common info pointer */
 
 	/* state fields */
 	int currentX;           /* current X position */
@@ -327,7 +317,7 @@ struct _WacomDeviceRec
 	int accel;              /* relative mode acceleration */
 	int numScreen;          /* number of configured screens */
 	int currentScreen;      /* current screen in display */
-	tvMode twinview;	/* using twinview mode of gfx card */
+	int twinview;	        /* using twinview mode of gfx card */
 	int tvoffsetX;		/* X edge offset for TwinView setup */
 	int tvoffsetY;		/* Y edge offset for TwinView setup */
 	int tvResolution[4];	/* twinview screens' resultion */
@@ -350,7 +340,8 @@ struct _WacomDeviceRec
  * WacomDeviceState
  *****************************************************************************/
 
-#define MAX_SAMPLES 4
+#define MAX_SAMPLES	20
+#define DEFAULT_SAMPLES 4
 
 #define PEN(ds)         ((((ds)->device_id) & 0x07ff) == 0x0022 || \
                          (((ds)->device_id) & 0x07ff) == 0x0042 || \
@@ -367,6 +358,7 @@ struct _WacomDeviceRec
 
 struct _WacomDeviceState
 {
+	LocalDevicePtr local;
 	int device_id;		/* tool id reported from the physical device */
 	int device_type;
 	unsigned int serial_num;
@@ -458,18 +450,13 @@ struct _WacomDeviceClass
 
 #define DEVICE_ISDV4 		0x000C
 
-#define ROTATE_NONE XWACOM_VALUE_ROTATE_NONE
-#define ROTATE_CW XWACOM_VALUE_ROTATE_CW
-#define ROTATE_CCW XWACOM_VALUE_ROTATE_CCW
-#define ROTATE_HALF XWACOM_VALUE_ROTATE_HALF
-
 #define MAX_CHANNELS 2
 
 struct _WacomCommonRec 
 {
 	char* wcmDevice;             /* device file name */
-	int wcmSuppress;             /* transmit position on delta > supress */
 	unsigned char wcmFlags;      /* various flags (handle tilt) */
+	int debugLevel;
 	int tablet_id;		     /* USB tablet ID */
 	int fd;                      /* file descriptor to tablet */
 	int fd_refs;                 /* number of references to fd; if =0, fd is invalid */
@@ -498,8 +485,7 @@ struct _WacomCommonRec
 	int npadkeys;                /* number of pad keys in the above array */
 	int padkey_code[MAX_BUTTONS];/* hardware codes for buttons */
 
-	LocalDevicePtr* wcmDevices;  /* array of devices sharing same port */
-	int wcmNumDevices;           /* number of devices */
+	WacomDevicePtr wcmDevices;   /* list of devices sharing same port */
 	int wcmPktLength;            /* length of a packet */
 	int wcmProtocolLevel;        /* 4 for Wacom IV, 5 for Wacom V */
 	float wcmVersion;            /* ROM version */
@@ -519,6 +505,8 @@ struct _WacomCommonRec
 	int wcmMaxCursorDist;	     /* Max mouse distance reported so far */
 	int wcmCursorProxoutDist;    /* Max mouse distance for proxy-out max/256 units */
 	int wcmCursorProxoutDistDefault; /* Default max mouse distance for proxy-out */
+	int wcmSuppress;        	 /* transmit position on delta > supress */
+	int wcmRawSample;	     /* Number of raw data used to filter an event */
 
 	int bufpos;                        /* position with buffer */
 	unsigned char buffer[BUFFER_SIZE]; /* data read from device */
@@ -629,5 +617,8 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds);
 Bool xf86WcmPointInArea(WacomToolAreaPtr area, int x, int y);
 Bool xf86WcmAreaListOverlap(WacomToolAreaPtr area, WacomToolAreaPtr list);
 
+/* Change pad's mode according to it core event status */
+int xf86WcmSetPadCoreMode(LocalDevicePtr local);
+
 /****************************************************************************/
 #endif /* __XF86WACOM_H */
